# Test code that prevent Ctrl + C
# against a little mess with files,
# for example when manifest.json replaced to minified version for some browser
# https://unix.stackexchange.com/a/48432/34318
trap "exit" INT


CHROMIUM_FILEPATH_TEST=~/extension-deployment-package.chromium.test.zip
FIREFOX_FILEPATH_TEST=~/extension-deployment-package.firefox.test.zip

CHROMIUM_FILEPATH_PROD=~/extension-deployment-package.chromium.prod.zip
FIREFOX_FILEPATH_PROD=~/extension-deployment-package.firefox.prod.zip
# Prod and test archives for case when I will need to add hotfix without using git branches and stash

FILES='*.seded.html *.min.js *.min.css images/logo/*.png images/logo/logo-with-name.svg audio/* manifest.json _locales'
VERSION=$(python3 -c "import datetime; print(datetime.datetime.now().strftime('%Y.%-m.%-d.%-H%M'))")

sed -e 's/.js/.min.js/g' < eventPage.html > eventPage.seded.html

perl -p -e 's/contentScript.js/contentScript.min.js/g;
            s/.css/.min.css/g' < eventPage.js > eventPage.seded.js

# Not * because names for external resources from Google must not be replaced
perl -p -e 's/.js/.min.js/g;
            s/popup.css/popup.min.css/g;
            s/progress.css/progress.min.css/g;
            s/snackbar.css/snackbar.min.css/g' < popup.html > popup.seded.html

mv manifest.json manifest.original.json

# Alter manifest for Firefox first because it is simpler to restore 'incognito' than 'application'
python3 -c "
import json
import sys

with open('manifest.original.json') as file:
    json_manifest = json.load(file)

# Version example: '2018.2.1.1459'
# '-' in usual %<letter> drop leading zero if exists -
# with leading zero can not publish for Chrome Web Store.
# Unix only
# https://stackoverflow.com/questions/904928/python-strftime-date-without-leading-0
# https://developer.chrome.com/extensions/manifest/version
json_manifest['version'] = sys.argv[1]

# Firefox still does not support it; set to true against warning
del json_manifest['background']['persistent']

with open('manifest.json', 'w') as file:
    file.write(json.dumps(json_manifest))
" $VERSION
sed -i '' 's/.html/.seded.html/g' manifest.json
python3 -m csscompressor popup.css > popup.min.css
python3 -m csscompressor progress.css > progress.min.css
python3 -m csscompressor snackbar.css > snackbar.min.css

# Commented temporally to check publihing process in ChromeStore without minified scripts
# cat eventPage.seded.js > eventPage.min.js
# cat popup.js > popup.min.js
# cat common.js > common.min.js
# cat contentScript.js > contentScript.min.js
uglifyjs eventPage.seded.js --compress drop_console=true --mangle --toplevel --verbose > eventPage.min.js
uglifyjs popup.js --compress drop_console=true --mangle --toplevel --verbose > popup.min.js
uglifyjs common.js --compress drop_console=true --mangle --verbose > common.min.js
uglifyjs contentScript.js --compress drop_console=true --mangle --toplevel --verbose > contentScript.min.js
# This is uglify-es, not uglify-js

if [[ $@ == 'prod' ]]; then
    rm $FIREFOX_FILEPATH_PROD  # remove because maybe some file was deleted
    zip -r $FIREFOX_FILEPATH_PROD $FILES
else
    rm $FIREFOX_FILEPATH_TEST
    zip -r $FIREFOX_FILEPATH_TEST $FILES
fi


python3 -c "
import json
import sys

with open('manifest.json') as file:
    json_manifest = json.load(file)
    json_manifest.pop('applications')

    json_manifest['incognito'] = 'not_allowed'
    json_manifest['background']['persistent'] = False
    json_manifest['container'] = 'GOOGLE_DRIVE'  # Chromium only

    # Quote from https://docs.python.org/3/library/sys.html:
    # The list of command line arguments passed to a Python script.
    # argv[0] is the script name (it is operating system dependent whether this is a full pathname or not).
    # If the command was executed using the -c command line option to the interpreter,
    # argv[0] is set to the string '-c'.
    # If no script name was passed to the Python interpreter, argv[0] is the empty string.
    if len(sys.argv) == 1 or sys.argv[1] != 'prod':
        json_manifest['name'] = 'Intelligent Speaker [QA]'

with open('manifest.json', 'w') as file:
    file.write(json.dumps(json_manifest))
" $@
if [[ $@ == 'prod' ]]; then
    rm $CHROMIUM_FILEPATH_PROD
    zip -r $CHROMIUM_FILEPATH_PROD $FILES
else
    rm $CHROMIUM_FILEPATH_TEST
    zip -r $CHROMIUM_FILEPATH_TEST $FILES
fi

git archive --output ~/extension-deployment-package.original-for-opera-review.zip HEAD -9

rm manifest.json
mv manifest.original.json manifest.json

rm *.seded.*
rm *.min.*
