/* TODO Code in this script duplicated in popup and event page,
 * better to move all of this functions to the event page and call from popup using messages.
 */


const stageUrl =  'https://mjqj47yt17.execute-api.us-east-1.amazonaws.com/test/',  // eslint-disable-line no-unused-vars
    stageUrlNew = 'https://m08srfkly0.execute-api.us-east-1.amazonaws.com/prod/',
    clientId = '722jaa8aarrubh9lc1v3ff2ibd',
    callbackUrl = 'https://e7fwq159ke.execute-api.us-east-1.amazonaws.com/prod/auth-exchange-code-to-access-token';
// update callback url at https://console.aws.amazon.com/cognito/users?region=us-east-1#/pool/us-east-1_nWJoGJQYE/app-integration-app-settings?_k=6icxd8

function goToAuth() {  // eslint-disable-line no-unused-vars
    // random value like 326263189424723457354115822995 - client will use it for first POST with exchanging to user_id
    const state = String(crypto.getRandomValues(new Uint32Array(3))).replace(/,/g, '');
    chrome.storage.sync.set({state, 'playbackRate': 1}, () =>
        // https://developer.chrome.com/extensions/tabs#method-create
        // http://docs.aws.amazon.com/cognito/latest/developerguide/cognito-user-pools-ux.html
        chrome.tabs.create({'url': 'https://intelligentspeaker.auth.us-east-1.amazoncognito.com/signup?' +
            '&response_type=code' +
            '&client_id=' + clientId +
            '&redirect_uri=' + callbackUrl +
            '&state=' + state})
    );
}


function _getMessages(lang) {  // eslint-disable-line no-unused-vars
    return fetch(chrome.runtime.getURL(`_locales/${lang}/messages.json`))
        .then(resp => resp.json());
}


function _getActiveTab(callback) {  // eslint-disable-line no-unused-vars
    chrome.tabs.query({'active': true}, tabs => {
        find: for (const tab of tabs) {
            if (tab.url) {
                // Opened devtools do not has url.
                // Not only during development but user can also has an opened devtools.
                // Tested in Google Chrome.
                callback(tab);
                break find;
            }
            console.error(
                'Not found active tabs. Defect of Chromium? I faced with this only when devtools opened');
        }

    });
}


/**
 * This function is the decorator for any calls where id_token is used.
 *
 * id_token is valid for one hour.
 *
 * @see {@link http://docs.aws.amazon.com/cognito/latest/developerguide/token-endpoint.html}
 * @see {@https://stackoverflow.com/questions/46491811}
 */
async function _keepTokenFresh(storage) {  // eslint-disable-line no-unused-vars
    console.log('_keepTokenFresh, storage:', storage);
    if (_getValueFromIdToken('exp', storage) < (Date.now() / 1000 | 0)) {

        try {
            const resp = await fetch(
                'https://intelligentspeaker.auth.us-east-1.amazoncognito.com/oauth2/token', {
                    'method': 'POST',
                    'headers': {
                        'Content-Type': 'application/x-www-form-urlencoded'
                    },
                    'body': `grant_type=refresh_token&client_id=${clientId}&refresh_token=${storage['refresh_token']}`});
            const json = await resp.json();
            const jsonTokenNew = {'id_token': json['id_token']};

            console.debug('New id token:', jsonTokenNew['id_token']);

            chrome.storage.sync.set(jsonTokenNew);

            return {...storage, ...jsonTokenNew};  // Merge
        } catch(e) {
            console.log(
                'TODO maybe normal case when no internet connection - handle this without errors');
            alert('Intelligent Speaker: error, try later');  // TODO i18n
            stop();
        }
    } else {
        return storage;
    }
}


async function _getInitData(state, isInit) {  // eslint-disable-line no-unused-vars
    let initData = {};

    const client = navigator.userAgent.match(/Chrome|OPR|Firefox/g).pop().toLowerCase();

    try {
        const resp = await fetch(`${stageUrlNew}init?state=${state}&client=${client}`);
        initData = await resp.json();
        console.log('Init data:', initData);
    } catch(e) {
        console.error(e);
        alert('Intelligent Speaker: error, try later');  // TODO i18n
        stop();
    }
    if (!initData) {
        // Case: signup was not finished
        if (!isInit) {
            goToAuth();
            stop();
        }
        return;
    }

    chrome.storage.sync.set({
        'id_token': initData['id_token'],
        'refresh_token': initData['refresh_token'],
        ..._transformHistoryToTheFormatOfExtendion(initData['history']),
        'isContinuousPlaying': initData['isContinuousPlaying'] === 'true',
        'playbackRate': initData['playbackRate'],
        'updated': Date.now() / 1000 | 0
    }), _ => chrome.storage.sync.remove('state');

    const email = _getValueFromIdToken('email', initData);

    /**
    * Edit form at
    * https://docs.google.com/forms/d/1FoHUVP4kNgwMxBucTXzM5Nto2KhoioBUKgRoOGsbOwg/edit
    * Account: intelligentspeakerapp@gmail.com
    *
    * I tested - it enough to set uninstall url only once.
    * Reloading of the extension and restarting of the OS did not reset it.
    */
    chrome.runtime.setUninstallURL(
        `https://docs.google.com/forms/d/e/1FAIpQLSdDNyd3COJzZEw9sHXE34Y2Ercrq0klL1hCn0Or2mZapG7UQw/viewform?usp=pp_url&entry.839337160&entry.1314511419=${email}`
    );

    return initData;
}
/**
 * TODO use the same structure in the database and extension,
 * think about that after reusing audio chunks with all users.
 */
function _transformHistoryToTheFormatOfExtendion(history) {
    const obj = {};
    history.forEach(item =>
        obj[item['utc']] = {
            'hashArticle': item['hash_article'],
            'lenChunks': item['len_chunks'],
            'title': item['title'],
            'url': item['url_article'],
            'favIconUrl': item['favIconUrl'],
            'isNew': item['isNew']
        }
    );
    return obj;
}



function _getValueFromIdToken(key, storage) {
    console.log('_getValueFromIdToken(): key:', key, 'storage: ', storage);
    const idTokenPayloadStr = storage['id_token'].split('.')[1].replace('_', '/');
    // Without replace() window.atob() can not decode base64
    // We do not store in chrome storage version after replace() because ApiGateway need original JWT (error otherwise)
    return JSON.parse(window.atob(idTokenPayloadStr))[key];
}
