console.log('args:', args); // eslint-disable-line no-undef

if (!window.isEventListenerAdded)
    window.isEventListenerAdded = false;

/* tabId send only when from context menu -
 * for later send message to this tab on finish of the adding to the feed
 * for ending of progress animation.
 */
if (args['tabId'])  // eslint-disable-line no-undef
    _progressShow();

/**
 * Extracted from https://raw.githubusercontent.com/JensPLarsen/ChromeExtension-GoogleDocsUtil/master/googleDocsUtil.js
 *
 * Class declared on the top becase will not hoisted, see
 * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Classes
 */

//Unnamed class because this content script may be injected few times - but class may be declared only once
var GoogleDocs = class {
    //https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Classes
    constructor() {
        this.textInDOM = '';
        this.selectedText = '';
        for (const paragraphrenderer of document.getElementsByClassName('kix-paragraphrenderer')) {
            this.textInDOM += paragraphrenderer.innerText;
            for (const lineview of paragraphrenderer.getElementsByClassName('kix-lineview')) {
                for (const selectionOverlay of lineview.getElementsByClassName('kix-selection-overlay')) {
                    const selectionOverlayRect = selectionOverlay.getBoundingClientRect();
                    for (const wordhtmlgeneratorWordNode of document.getElementsByClassName('kix-wordhtmlgenerator-word-node')) {
                        const nodeText = this.cleanDocumentText(wordhtmlgeneratorWordNode.innerText);
                        const wordhtmlgeneratorWordNodeRect = wordhtmlgeneratorWordNode.getBoundingClientRect();
                        if (this.doesRectsOverlap(wordhtmlgeneratorWordNodeRect, selectionOverlayRect)) {
                            const selectionStartIndex = this.getLocalCaretIndex(
                                selectionOverlayRect.left - wordhtmlgeneratorWordNodeRect.left, wordhtmlgeneratorWordNode, lineview);
                            const selectionEndIndex = this.getLocalCaretIndex(
                                selectionOverlayRect.left + selectionOverlayRect.width - wordhtmlgeneratorWordNodeRect.left, wordhtmlgeneratorWordNode, lineview);
                            this.selectedText += nodeText.substring(selectionStartIndex, selectionEndIndex);
                        }
                    }
                }
            }
        }
    }


    //http://stackoverflow.com/questions/306316/determine-if-two-rectangles-overlap-each-other
    doesRectsOverlap(RectA, RectB) {
        return RectA.left <= RectB.right && RectA.right >= RectB.left &&
            RectA.top <= RectB.bottom && RectA.bottom >= RectB.top;
    }


    /** Gets the caret index on the innerText of the element.
      * @param caretX: The x coordinate on where the element the caret is located
      * @param element: The element on which contains the text where in the caret position is
      * @param simulatedElement: Doing the calculation of the caret position, we need to create a temporary DOM, the DOM will be created as a child to the simulatedElement.
      */
    getLocalCaretIndex(caretX, element, simulateElement) {

        //Creates a span DOM for each letter
        const text = this.cleanDocumentText(element.innerText);
        const container = document.createElement('div');
        const letterSpans = [];
        for (const txt of text) {
            const textNode = document.createElement('span');
            textNode.innerText = txt;
            textNode.style.cssText = element.style.cssText;
            //"pre" = if there are multiple white spaces, they will all be rendered. Default behavior is for them to be collapesed
            textNode.style.whiteSpace = 'pre';
            letterSpans.push(textNode);
            container.appendChild(textNode);
        }
        simulateElement.appendChild(container);

        //The caret is usually at the edge of the letter, we find the edge we are closest to.
        let index = 0;
        let currentMinimumDistance = -1;
        const containerRect = container.getBoundingClientRect();
        for (let i = 0; i < letterSpans.length; i++) {
            const rect = letterSpans[i].getBoundingClientRect();
            const left = rect.left - containerRect.left;
            const right = left + rect.width;
            if (currentMinimumDistance == -1) {
                currentMinimumDistance = Math.abs(caretX - left);
            }
            const leftDistance = Math.abs(caretX - left);
            const rightDistance = Math.abs(caretX - right);

            if (leftDistance <= currentMinimumDistance) {
                index = i;
                currentMinimumDistance = leftDistance;
            }

            if (rightDistance <= currentMinimumDistance) {
                index = i + 1;
                currentMinimumDistance = rightDistance;
            }
        }

        //Clean up
        container.remove();
        return index;
    }


    /** Google Docs like to add \u200B and non breaking spaces to make sure the browser shows the text correct. 
      * When getting the text, we would prefer to get clean text.
      */
    cleanDocumentText(text) {
        let cleanedText = text.replace(/\u200B/g, '');
        const nonBreakingSpaces = String.fromCharCode(160);
        const regex = new RegExp(nonBreakingSpaces, 'g');
        cleanedText = cleanedText.replace(regex, ' ');
        return cleanedText;
    }

    /*
     * Query selector of <script nonce=''> because after page loading
     * I not found full text neither in global variables nor at the DOM text.
     * '.kix-paragraphrenderer' contains only rendered texts -
     * Google Doc will not insert to the DOM huge book with hundreds of pages.
     *
     * TODO now if user removed part of text - will be added original document,
     * that was loaded from Google server on page opening.
     * https://stackoverflow.com/questions/49273466/google-doc-how-to-get-full-text-through-the-browser-extension
     * */
    getFullText() {
        let textLoadedOnDocumentOpening = '';

        /** Define this variables here because will be blocker error when
         *  DOCS_modelChunkLoadStart = some value, in model, without 'var' or 'let' or 'const'.
         *
         *  _getTimingInstance() exists in the scope of Google Doc but
         *  extension works in isolated world where only DOM shared.
         *  Not remove this code from innerText because maybe user want to listen it :) */
        let DOCS_modelChunk = undefined;
        let DOCS_modelChunkLoadStart = undefined;  // eslint-disable-line
        let DOCS_modelChunkParseStart = undefined;  // eslint-disable-line

        // Without parentheses will be syntax error because '{' is like a start of the function body
        const _getTimingInstance = _ => ({'incrementTime': _ => {}});  // eslint-disable-line
        // Document text in the source code even with disabled JS, yeah name 'model'
        for (const node of document.querySelectorAll('[nonce]'))
            if (node.innerText.startsWith('DOCS_modelChunk =')) {
                // Drop loadModelChunk() against adding this text to the visible area.
                eval(node.innerText.replace(
                    'DOCS_warmStartDocumentLoader.loadModelChunk(DOCS_modelChunk); DOCS_modelChunk = undefined',
                    ''
                ));
                textLoadedOnDocumentOpening += DOCS_modelChunk[0]['s'];

                // For garbage collector, big object
                DOCS_modelChunk = undefined;
            }
        return textLoadedOnDocumentOpening.length < this.textInDOM.length ?
            this.textInDOM : textLoadedOnDocumentOpening;
    }
};


/** In usual js that code must be a function getSelection() but this is a content script that executed
* every time when user want listen the page - so this is not a function against redeclaration and for
    * similarity with execution flow. */
var selected = window.getSelection().toString();  // var but not let becauce can redeclare
if (!selected) {  // try to get selected from iframes
    for (let iframe of document.getElementsByTagName('iframe')) {
        try {
            var _selected = iframe.contentDocument.getSelection().toString();
            /** Not `let` because `var` can be redeclared - because if user want to listen page second time -
             * this script will be injected (executed) second time and all top-level or `var` variables will remain
             * from previous execution. */
        } catch (e) {
            /** `Uncaught DOMException: Failed to read the 'contentDocument' property from 'HTMLIFrameElement':
                * Blocked a frame with origin "http://www.foxnews.com" from accessing a cross-origin frame.`
                * For example - embeded Instagram.
                * http://www.foxnews.com/lifestyle/2017/07/23/this-woman-lost-99-pounds-without-following-diet.html
                * There is also another way to handle such case: check src of every frame before query -
                * domains must match. But try...catch is better because without IF? :) */
            continue;
        }
        if (_selected) {
            selected = _selected;
            break;
        }
    }
}
/** About selection: on the page with iframe user can select text both in the main document and inside iframe,
    * current code will get selection from main document first. For example Evernote. Add special handligh for Evernote -
    * if found selection inside iframe with note - omit selection from main document? This sound logic - user can
    * accidently remain selection of title with desired selection of the part of big note. */
console.log('contentScript.js; selected:', selected);

if (selected.trim()) {
    /* TODO here I can detect language of selected text with
     * chrome.i18n.detectLanguage(str, callback)
     * https://developer.chrome.com/extensions/i18n
     *
     * In current scope we have `args` variable with `lang` inside (from popup or event page)
     * but may be a case when page in multiple languages
     * and user select text written in minority language.
     */
    sendMessage(selected.trim());
} else { // extract text of the article from html

    if (window.location.href.match(/.+\.[m.]?wikipedia.org\/.+/)) {  // `[m.]` to handle mobile version too
        /** Use api to extract plaintext - against [0], [edit], [1] and against block with References;
         *    not at Lambda to not setup VPC for access to the internet (and work on client is cheaper for us).
         */
        const subdomain = window.location.host.split('.')[0];
        const pageId = document.documentElement.outerHTML.match(/"wgArticleId":\d+/)[0].split(':')[1];
        fetch(`https://${subdomain}.wikipedia.org/w/api.php?action=query&prop=extracts&explaintext&format=json&origin=*&pageids=${pageId}`)
            .then(r => r.json())
            .then(r => {
                // replace() to remove === HEADERS ===
                sendMessage(r['query']['pages'][pageId]['extract'].replace(/=/g, ''));
            });
    } else if (window.location.href.startsWith('https://docs.google.com/document/d')) {
        /** Href not just docs.google.com/document because
         *  published document in plain html have href docs.google.com/document/u/xxxxxxxxx... */
        const googleDocs = new GoogleDocs();
        sendMessage(googleDocs.selectedText || googleDocs.getFullText());
    //} else if (window.location.href.match(/https:\/\/www.reddit.com\/r\/\w+\/comments/)) {
        //Download on the client because as I read - fetching can take a minute for 500 comments - money of Lambda
        //TODO move to the Python - can not works from browser because now not possible to set User-Agent
        //https://www.reddit.com/r/redditdev/comments/2pe7fi/is_my_code_wrong_or_is_reddit_blocking_requests/
        //https://bugs.chromium.org/p/chromium/issues/detail?id=571722
        // const device_id = String(crypto.getRandomValues(new Uint32Array(3))).replace(/,/g, '');
        // const client_id = 'ebtOPUwW0_o7eA'; //From https://www.reddit.com/prefs/apps

        // const callback = access_token => fetch('https://oauth.reddit.com/r/UpliftingNews/comments/7ewkqa', {
        //     'headers': {
        //         'Authorization': access_token,
        //         'User-Agent': 'intelligent-speaker'
        //     }
        // })
        //     .then(resp => resp.json())
        //     .then(resp => console.log(resp));

        // fetch('https://www.reddit.com/api/v1/access_token', {
        //     'method': 'POST',
        //     'headers': {
        //         'Authorization': `basic ${window.btoa(client_id + ':')}`,
        //         'Content-Type': 'application/x-www-form-urlencoded'
        //         //Without implisit content type will be default json and answer will be `unsupported_grant_type`
        //     }, 'body': 
        //         `grant_type=https://oauth.reddit.com/grants/installed_client&device_id=${device_id}`
        // })
        //     .then(resp => resp.json())
        //     .then(resp => {
        //         window.localStorage.setItem('reddit',
        //             JSON.stringify(
        //                 //Without stringify will be saved '[object object]'
        //                 {
        //                     'access_token': resp.access_token,
        //                     'expires_in': Date.now() / 1000 | 0 + resp.expires_in,
        //                     'device_id': device_id
        //                 }
        //             )
        //         );
        //         callback(resp.access_token);
        //     });

    } else {
        sendMessage(document.documentElement.outerHTML);
    }
}


function sendMessage(stringForVoicing) {
    args['stringForVoicing'] = stringForVoicing;  // eslint-disable-line no-undef
    chrome.runtime.sendMessage(args);  // eslint-disable-line no-undef
}


/**
 * TODO rewrite to using WebComponents and Shadow DOM when it will be supported by Firefox
 * https://caniuse.com/#feat=shadowdomv1
**/
function _progressShow() {

    // TODO refactor names to be in sync with snackbar, drop usage of the id
    const progress = document.createElement('random-element-name-against-collision-with-css-of-page');
    progress.id = 'progress--this-is-some-random-id-name-that-do-not-collide-with-existing-id-in-dom';

    const indeterminate = document.createElement('random-element-name-against-collision-with-css-of-page');
    indeterminate.classList.add(
        'indeterminate--this-is-some-random-class-name-that-do-not-collide-with-existing-class-in-dom');

    progress.appendChild(indeterminate);

    document.body.insertBefore(progress, null);

    // Add 'active' class after creading of node - for triggering CSS animation
    setTimeout(_ =>
        progress.classList.add(
            'active--this-is-some-random-class-name-that-do-not-collide-with-existing-class-in-dom'),
    100);

    /* I tried   chrome.runtime.hasEventListeners() but it only for Chromium,
     * Firefox and Chromium has hasEventListener() but it return false even when listener added. */
    !window.isEventListenerAdded &&
        chrome.runtime.onMessage.addListener(message => {
            console.log('Event listener of the tab');

            window.isEventListenerAdded = true;

            /* Do not reuse existing 'progress' variable because this closure remember first one
             * and on second adding to the feed from context menu will try to operate with deleted.
             */
            const progressCurrent = document.getElementsByTagName(
                'random-element-name-against-collision-with-css-of-page')[0];

            progressCurrent.classList.remove(
                'active--this-is-some-random-class-name-that-do-not-collide-with-existing-class-in-dom');

            /*
             * I tried 'getComputedStyle(progress).transitionDuration' but
             * I do not know why but multiplication of that result behaved like zero
             * and fadeout was without animation.
             */
            setTimeout(
                _ => progressCurrent.remove(),
                // Do not polute foreing DOM - it may break functionality and view
                500
            );

            _showSnackbar(message['snackbar--html']);  // eslint-disable-line no-undef
        });
}

function _showSnackbar(html) {
    if (!html)
        return;
    // Now we do not show snackbar in case of errors (when alert showed)

    console.log(arguments.callee.name, arguments);

    const snackbar = document.createElement('intelligentspeaker--snackbar--случайное-имя-против-коллизий');
    snackbar.innerHTML = html;
    document.body.insertBefore(snackbar, null);

    setTimeout(_ =>
        snackbar.classList.add('intelligentspeaker--snackbar--class_active--случайное-имя-против-коллизий'),
    9);
    // Without setTimeout will appear without fadeit subtle animation, looks like for CSS init

    setTimeout(_ => {
        snackbar.classList.remove('intelligentspeaker--snackbar--class_active--случайное-имя-против-коллизий');
        setTimeout(_ => snackbar.remove(), 500);
        // Do not polute foreing DOM - it may break functionality and view
    }, 3000);
}
